/* IBM Confidential
 * OCO Source Materials
 * 5724-N40, 5724-N41, 5724-N42, 5724-N43, 5724-L77, 5724-L78, 5724-L79, 5724-L80, 5724-L81, 5724-L82
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.migration.sap.v602_to_v610;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ibm.j2ca.migration.ArtifactChangeArguments;
import com.ibm.j2ca.migration.Change;
import com.ibm.j2ca.migration.IChangeArguments;
import com.ibm.j2ca.migration.MigrationException;
import com.ibm.j2ca.migration.changedata.wbi.ConvertWBIASItoJCAASI;
import com.ibm.j2ca.migration.internal.changes.wbi.ConvertWBIASItoJCAChange;
import com.ibm.j2ca.migration.sap.v602_to_v610.MigrationMessages;
import com.ibm.j2ca.migration.util.AttributeMap;
import com.ibm.j2ca.migration.util.SearchHelper;

public class AddqRFCQueueNameASIForSAPChange extends Change {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2008."; //$NON-NLS-1$
	
	private IFile file;
	protected ArrayList<IFile> topLevelBOs = null;
	
	public AddqRFCQueueNameASIForSAPChange(IFile file, AddqRFCQueueNameASIForSAP changeData) {
		super(changeData);
		this.file = file;
	}
	
	public AddqRFCQueueNameASIForSAP getChangeData() { return (AddqRFCQueueNameASIForSAP)super.getChangeData(); }
	
	public String getChangeDetails() {
		return MigrationMessages.AddqRFCQueueNameASIForSAP_Description;
	}
	
	public IChangeArguments getChangeArguments() {
		ArtifactChangeArguments argument = new ArtifactChangeArguments(file);
		return argument;
	}
	
	public void perform(IProgressMonitor pm) throws Exception {
		if(file.exists()) {
			DOMParser parser = new DOMParser();
			parser.parse(file.getLocation().toString());			
			Document doc = parser.getDocument();
			
			NodeList sapALEBusinessObjectTypeMetadatas = doc.getElementsByTagNameNS("*", "sapALEBusinessObjectTypeMetadata");
			if(sapALEBusinessObjectTypeMetadatas==null || sapALEBusinessObjectTypeMetadatas.getLength()==0)
				return; // Not ALE API
			NodeList types = ((Element) sapALEBusinessObjectTypeMetadatas.item(0)).getElementsByTagNameNS("*", "Type");
			if(types==null || types.getLength()==0)
				return; // Not ALE API
			if(!new String("IDOC").equalsIgnoreCase(types.item(0).getTextContent()))
				return; // Not ALE API
			
			NodeList sequences = doc.getElementsByTagNameNS("*", "sequence");
			if(sequences==null || sequences.getLength()==0)
				return;
			NodeList elements = ((Element) sequences.item(0)).getElementsByTagNameNS("*", "element");
			if(elements!=null)
				for(int i=0; i<elements.getLength(); i++) {
					Element element = (Element) elements.item(i);
					if(element.hasAttribute("name") && element.getAttribute("name").equalsIgnoreCase("qRFCQueueName"))
						return; // No need to add one more "qRFCQueueName"
				}
			
			//Create Element "qRFCQueueName"
			Element qRFCQueueName = doc.createElement("element");
			qRFCQueueName.setAttribute("name", "qRFCQueueName");
			qRFCQueueName.setAttribute("type", "string");
			qRFCQueueName.setAttribute("nillable", "true");
			qRFCQueueName.setAttribute("minOccurs", "1");
			qRFCQueueName.setAttribute("maxOccurs", "1");
			sequences.item(0).appendChild(qRFCQueueName);
			
			Element annotation = doc.createElement("annotation");
		    annotation.setAttribute("xml:space", "preserve");
		    qRFCQueueName.appendChild(annotation);
		    
		    Element appinfo = doc.createElement("appinfo");
		    appinfo.setAttribute("source", "http://www.ibm.com/xmlns/prod/websphere/j2ca/sap/metadata");
		    annotation.appendChild(appinfo);
			
		    Element sapALEPropertyTypeMetadata = doc.createElementNS("http://www.ibm.com/xmlns/prod/websphere/j2ca/sap/metadata", "sapALEPropertyTypeMetadata");
		    sapALEPropertyTypeMetadata.setPrefix("sapasi");
		    sapALEPropertyTypeMetadata.setAttribute("xmlns:sapasi", "http://www.ibm.com/xmlns/prod/websphere/j2ca/sap/metadata");
		    appinfo.appendChild(sapALEPropertyTypeMetadata);
		    
		    Element fieldName = doc.createElementNS("http://www.ibm.com/xmlns/prod/websphere/j2ca/sap/metadata", "FieldName");
		    fieldName.setPrefix("sapasi");
		    fieldName.setTextContent("qRFCQueueName");
		    sapALEPropertyTypeMetadata.appendChild(fieldName);
		    
		    Element segmentHierarchy = doc.createElementNS("http://www.ibm.com/xmlns/prod/websphere/j2ca/sap/metadata", "SegmentHierarchy");
		    segmentHierarchy.setPrefix("sapasi");
		    segmentHierarchy.setTextContent("0");
		    sapALEPropertyTypeMetadata.appendChild(segmentHierarchy);
		    
		    Element offSet = doc.createElementNS("http://www.ibm.com/xmlns/prod/websphere/j2ca/sap/metadata", "OffSet");
		    offSet.setPrefix("sapasi");
		    offSet.setTextContent("0");
		    sapALEPropertyTypeMetadata.appendChild(offSet);
		    
		    Element primaryKey = doc.createElementNS("http://www.ibm.com/xmlns/prod/websphere/j2ca/sap/metadata", "PrimaryKey");
		    primaryKey.setPrefix("sapasi");
		    primaryKey.setTextContent("false");
		    sapALEPropertyTypeMetadata.appendChild(primaryKey);
		    
		    Element foreignBOKeyRef = doc.createElementNS("http://www.ibm.com/xmlns/prod/websphere/j2ca/sap/metadata", "ForeignBOKeyRef");
		    foreignBOKeyRef.setPrefix("sapasi");
		    foreignBOKeyRef.setTextContent("");
		    sapALEPropertyTypeMetadata.appendChild(foreignBOKeyRef);
		    
		    Element maxLength = doc.createElementNS("http://www.ibm.com/xmlns/prod/websphere/j2ca/sap/metadata", "MaxLength");
		    maxLength.setPrefix("sapasi");
		    maxLength.setTextContent("0");
		    sapALEPropertyTypeMetadata.appendChild(maxLength);
		    
			writeXml(file, doc);
		}
	}
	

}
