/* IBM Confidential
 * OCO Source Materials
 * 5724-L79
 *  Copyright IBM Corporation 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.migration.sap.v602_to_v620;

import java.util.ArrayList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.NullProgressMonitor;

import com.ibm.j2ca.migration.IChange;
import com.ibm.j2ca.migration.MigrationException;
import com.ibm.j2ca.migration.changedata.RenameAdapter;
import com.ibm.j2ca.migration.changedata.ModuleChangeData.ModuleType;
import com.ibm.j2ca.migration.changedata.ServiceChangeData.ServiceType;
import com.ibm.j2ca.migration.sap.v602_to_v610.MigrationMessages;
import com.ibm.j2ca.migration.util.SearchHelper;
import com.ibm.j2ca.migration.util.Util;
import com.ibm.j2ca.migration.validation.quickfix.MigrationMarkerResolution;

public class SAPAdapterNameFix extends MigrationMarkerResolution {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2007."; //$NON-NLS-1$
	
	public String getLabel() {
		return MigrationMessages.SAPAdapterNameFix_AdapterNameError_QuickFix; 
	}
	
	public void run(IMarker marker) {
		IResource resource = marker.getResource();
		IProject project = resource.getProject();
		
		ArrayList<IFile> serviceDescriptorFiles = SearchHelper.getContainerFiles(project, "(.*\\.import)|(.*\\.export)", "");
		for(IFile file : serviceDescriptorFiles) {			
			RenameAdapter renameAdapter = new RenameAdapter("IBM WebSphere Adapter for SAP Software with transaction support", new ServiceType [] {ServiceType.IMPORT, ServiceType.EXPORT});
			ArrayList<IChange> changes = renameAdapter.createChanges(file, new ModuleType [] {ModuleType.INBOUND, ModuleType.OUTBOUND});
			for(IChange change : changes) {
				try {
					change.execute(new NullProgressMonitor());
				}
				catch(MigrationException e) {
					Util.writeLog(e);
				}
			}
		}
	}
}
