/* IBM Confidential
 * OCO Source Materials
 * 5724-L79
 *  Copyright IBM Corporation 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.migration.sap.v602_to_v620;

import java.util.ArrayList;

import com.ibm.j2ca.migration.MigrationException;
import com.ibm.j2ca.migration.ModuleMigrationTask;
import com.ibm.j2ca.migration.changedata.*;
import com.ibm.j2ca.migration.changedata.ModuleChangeData.ModulePath;
import com.ibm.j2ca.migration.changedata.ModuleChangeData.ModuleType;
import com.ibm.j2ca.migration.changedata.ServiceChangeData.ServiceType;
import com.ibm.j2ca.migration.sap.v602_to_v610.AddqRFCQueueNameASIForSAP;
import com.ibm.j2ca.migration.util.Util;

public class SAPModuleMigrationTask extends ModuleMigrationTask {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2007."; //$NON-NLS-1$
	public static final String SAP_BUNDLE_NAME = "com.ibm.j2ca.migration.sap"; //$NON-NLS-1$
	public String sapAPI = "";
	
	public static ArrayList<String> dataBindingTypes = new ArrayList<String>();
	static { 
//		dataBindingTypes.add("com.ibm.j2ca.sap.emd.runtime.SAPBAPIDataBindingGenerator");
//		dataBindingTypes.add("com.ibm.j2ca.sap.emd.runtime.SAPIDocDataBindingGenerator");
//		dataBindingTypes.add("com.ibm.j2ca.sap.emd.runtime.SAPSQIDataBindingGenerator");
//		dataBindingTypes.add("com.ibm.j2ca.sap.emd.runtime.SAPRFCServerDataBindingGenerator");
	}
	private boolean isReusedByWBIA2JCAMigrationModule = false;
	
	public boolean isReusedByWBIA2JCAMigrationModule() {
		return isReusedByWBIA2JCAMigrationModule;
	}

	public void setReusedByWBIA2JCAMigrationModule(
			boolean isReusedByWBIA2JCAMigrationModule) {
		this.isReusedByWBIA2JCAMigrationModule = isReusedByWBIA2JCAMigrationModule;
	}

	public SAPModuleMigrationTask(boolean reusedByWBIA2JCAMigrationModule) {
		super();
		this.isReusedByWBIA2JCAMigrationModule = reusedByWBIA2JCAMigrationModule;
	}
	
	public SAPModuleMigrationTask() {
		super();
	}
	
	public ArrayList<ModuleChangeData> getChangeData() throws MigrationException {
		ArrayList<ModuleChangeData> changeDataList = new ArrayList<ModuleChangeData>();
		
		if(isReusedByWBIA2JCAMigrationModule == true) {
		       if(sapAPI.startsWith("BAPI"))
		    	   dataBindingTypes.add("com.ibm.j2ca.sap.emd.runtime.SAPBAPIDataBindingGenerator");
		       else if(sapAPI.equalsIgnoreCase("ALE"))
		    	   dataBindingTypes.add("com.ibm.j2ca.sap.emd.runtime.SAPIDocDataBindingGenerator");
		} else {
			//CR93125 - clear the list. Avoid duplicate items if it's invoked for multiple times.
			dataBindingTypes.clear();
			
			   dataBindingTypes.add("com.ibm.j2ca.sap.emd.runtime.SAPBAPIDataBindingGenerator");
			   dataBindingTypes.add("com.ibm.j2ca.sap.emd.runtime.SAPIDocDataBindingGenerator");
			   dataBindingTypes.add("com.ibm.j2ca.sap.emd.runtime.SAPSQIDataBindingGenerator");
			   dataBindingTypes.add("com.ibm.j2ca.sap.emd.runtime.SAPRFCServerDataBindingGenerator");
			}
		
		changeDataList.add(new CreateBO("/resources/v602_to_v610/bo/PrimaryKeyPairType.xsd", MIGRATION_BUNDLE_NAME, false, new ModuleType [] {ModuleType.OUTBOUND}, ModulePath.ROOT));
		changeDataList.add(new CreateBO("/resources/v602_to_v610/bo/WBIFault.xsd", MIGRATION_BUNDLE_NAME, false, new ModuleType [] {ModuleType.OUTBOUND}, ModulePath.ROOT));
		changeDataList.add(new CreateBO("/resources/v602_to_v610/bo/DuplicateRecordFault.xsd", MIGRATION_BUNDLE_NAME, false, new ModuleType [] {ModuleType.OUTBOUND}, ModulePath.ROOT));
		changeDataList.add(new CreateBO("/resources/v602_to_v610/bo/MatchesExceededLimitFault.xsd", MIGRATION_BUNDLE_NAME, false, new ModuleType [] {ModuleType.OUTBOUND}, ModulePath.ROOT));
		changeDataList.add(new CreateBO("/resources/v602_to_v610/bo/MissingDataFault.xsd", MIGRATION_BUNDLE_NAME, false, new ModuleType [] {ModuleType.OUTBOUND}, ModulePath.ROOT));
		changeDataList.add(new CreateBO("/resources/v602_to_v610/bo/MultipleMatchingRecordsFault.xsd", MIGRATION_BUNDLE_NAME, false, new ModuleType [] {ModuleType.OUTBOUND}, ModulePath.ROOT));
		changeDataList.add(new CreateBO("/resources/v602_to_v610/bo/RecordNotFoundFault.xsd", MIGRATION_BUNDLE_NAME, false, new ModuleType [] {ModuleType.OUTBOUND}, ModulePath.ROOT));
		
		changeDataList.add(new RemoveASI("DataDelimiter", new ModuleType [] {ModuleType.OUTBOUND}));
		
		changeDataList.add(new ChangeConnectionAttributeValue("selectorType", "com.ibm.j2ca.sap.emd.runtime.SAPFunctionSelectorImpl", new ServiceType [] {ServiceType.EXPORT}));		
		changeDataList.add(new RenameFunction("Update", "Updatewithdelete", new ServiceType [] {ServiceType.IMPORT, ServiceType.EXPORT}));
		changeDataList.add(new ChangeESBBindingAttribute("dataBindingType", dataBindingTypes, new ServiceType [] {ServiceType.IMPORT, ServiceType.EXPORT}));
		changeDataList.add(new ImportConnectionProperty("ignoreBAPIReturn", "com\\.ibm\\.j2ca\\.sap\\.SAPInteractionSpec", new ServiceType [] {ServiceType.IMPORT}));
		
		changeDataList.add(new ChangeConnectionAttributeValue("type", "com.ibm.j2ca.sap.SAPRFCActivationSpec", "esbBinding/dataBindingType", "com\\.ibm\\.j2ca\\.sap\\.emd\\.runtime\\.SAPRFCServerDataBindingGenerator", new ServiceType [] {ServiceType.EXPORT}));
		changeDataList.add(new ChangeConnectionAttributeValue("interactionType", "com.ibm.j2ca.sap.SAPBapiInteractionSpec", "esbBinding/dataBindingType", "com\\.ibm\\.j2ca\\.sap\\.emd\\.runtime\\.SAPBAPIDataBindingGenerator", new ServiceType [] {ServiceType.IMPORT}));
		changeDataList.add(new ChangeConnectionAttributeValue("interactionType", "com.ibm.j2ca.sap.SAPAleInteractionSpec", "esbBinding/dataBindingType", "com\\.ibm\\.j2ca\\.sap\\.emd\\.runtime\\.SAPIDocDataBindingGenerator", new ServiceType [] {ServiceType.IMPORT}));
		changeDataList.add(new ChangeConnectionAttributeValue("interactionType", "com.ibm.j2ca.sap.SAPSQIInteractionSpec", "esbBinding/dataBindingType", "com\\.ibm\\.j2ca\\.sap\\.emd\\.runtime\\.SAPSQIDataBindingGenerator", new ServiceType [] {ServiceType.IMPORT}));
		
		//These changes are only required for JCA-2-JCA migration
		if(!isReusedByWBIA2JCAMigrationModule) {
		try {
			if(SAPConnectorMigrationTask.hasLocalTransaction(getMigrationContext())) {
				changeDataList.add( new RenameAdapter("IBM WebSphere Adapter for SAP Software with transaction support", new ServiceType [] {ServiceType.IMPORT, ServiceType.EXPORT}) );
			}
		}
		catch(Exception e) {
			Util.writeLog(e);
		}
		
		changeDataList.add(new AddqRFCQueueNameASIForSAP());
		}
		
		//changeDataList.add(new SAPAppendXPath());
		
		return changeDataList;
	}
}