/* IBM Confidential
 * OCO Source Materials
 * 5724-N40, 5724-N41, 5724-N42, 5724-N43, 5724-L77, 5724-L78, 5724-L79, 5724-L80, 5724-L81, 5724-L82
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.migration.sap.wbi_to_v610;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import com.ibm.j2ca.migration.internal.changes.CreateFileChange;
import com.ibm.j2ca.migration.sap.v602_to_v610.MigrationMessages;
import com.ibm.j2ca.migration.util.SearchHelper;
import com.ibm.j2ca.migration.util.Util;

public class CreateBOWrapperBGForSAPChange extends CreateFileChange {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2008."; //$NON-NLS-1$
	
	private IFile file;
	private String boName = null;
	private IProject project = null;
	private ArrayList<IFile> topLevelBOs = null;
	
	public CreateBOWrapperBGForSAPChange(IProject project, CreateBOWrapperBGForSAP changeData) {
		super(project, changeData);
		this.project = project;
	}
	
	public String getChangeDetails() {
		return MigrationMessages.CreateBOWrapperBGForSAPChange_Description;
	}
	
	public CreateBOWrapperBGForSAP getChangeData() { return (CreateBOWrapperBGForSAP)super.getChangeData(); }

	public void perform(IProgressMonitor pm) throws Exception {
		if(topLevelBOs == null)
			topLevelBOs = SearchHelper.getTopLevelBOs(project);
		
		if(topLevelBOs.size()>0) {
		   String targetFileLocation = getTargetLocation();
		   //CR92961 - Skip it if the target file already exists.
		   if(new File(targetFileLocation).exists()) { return; }
		   
		   copy(getChangeData().sourceUrl, targetFileLocation, true);
           getProject().refreshLocal(IResource.DEPTH_INFINITE, pm);
           for(IProject sharedProject : SearchHelper.getSharedProjects(project)) {
           	  sharedProject.refreshLocal(IResource.DEPTH_INFINITE, pm);
           }
		}
		
        /*
        if(beforeCreate(pm)) {
	        IFile targetFile = SearchHelper.getFile(getProject(), targetFileLocation);
	        afterCreate(targetFile, pm);
        }
        */
	}
	
	protected String getTargetLocation() {
		 
		if(topLevelBOs == null) {
			try { 
				topLevelBOs = SearchHelper.getTopLevelBOs(project);
				if(topLevelBOs != null)
					   return topLevelBOs.get(0).getParent().getLocation().toString() + "/" + getWrapperBGBOName() + ".xsd";

			} catch(Exception e) {
			}
		} else {
			return topLevelBOs.get(0).getParent().getLocation().toString() + "/" + getWrapperBGBOName() + ".xsd";
		}	
		 
		return project.getLocation().toString() + "/" + getWrapperBGBOName() + ".xsd";
	
		/*
		String fileLocation = getChangeData().sourceUrl.getFile();

		String fileName = null;
		try {
			fileName = getWrapperBGBOName() + ".xsd";
		}
		catch(Exception e) {
			fileName = fileLocation.substring(fileLocation.lastIndexOf('/'));
		}
		
		IContainer targetContainer = getTargetContainer();
		return targetContainer.getLocation().toString() + "/" + fileName;
		*/
		
	}
	
	protected String getBOName() throws IOException, SAXException {
		if(boName==null) {
			boName = Util.getBusinessObjectName(file).replaceAll("BG$", "");
		}
		return boName;
	}
	
	protected String getWrapperBOName() throws IOException, SAXException {
		return getBOName() + "Wrapper";
	}
	
	protected String getWrapperBGBOName() {
		//return getBOName() + "WrapperBG";
		if(getChangeData().sapAPI.startsWith("BAPI"))
			   return "BAPIWrapperBG";
		else if (getChangeData().sapAPI.equalsIgnoreCase("ALE"))
			   return "ALEWrapperBG";
		else return "WrapperBG";
	}
	
	public void afterCreate(IFile targetFile, IProgressMonitor pm) throws Exception {
        String baseNamespace = ((CreateBOWrapperBGForSAP)this.getChangeData()).baseNamespace;
        if(baseNamespace==null){
        	baseNamespace = Util.getBaseNamespace(getProject(), targetFile.getLocation().toString());        
        }
        String boWrapperNamespace = baseNamespace+"/"+getWrapperBOName().toLowerCase();
        
        DOMParser parser = new DOMParser();	
		parser.parse(targetFile.getLocation().toString());				
		Document doc = parser.getDocument();
		
		Element eSchema	= (Element)doc.getElementsByTagNameNS("*", "schema").item(0);
		if(eSchema!=null) {
			eSchema.setAttribute("targetNamespace", baseNamespace + "/" + getWrapperBGBOName().toLowerCase());
			eSchema.setAttribute("xmlns:" + getWrapperBOName(), boWrapperNamespace);
		}
		
		Element eImport = (Element)eSchema.getElementsByTagNameNS("*", "import").item(0);
		if(eImport!=null) {
			eImport.setAttribute("namespace", boWrapperNamespace);
			eImport.setAttribute("schemaLocation", getWrapperBOName()+".xsd");
		}
		
		Element eComplexType = (Element)eSchema.getElementsByTagNameNS("*", "complexType").item(0);
		if(eComplexType!=null) {
			eComplexType.setAttribute("name", getWrapperBGBOName());
		}
		
		Element eElement = (Element)eComplexType.getElementsByTagNameNS("*", "element").item(0);
		if(eElement!=null) {
			eElement.setAttribute("name", getWrapperBOName());
			eElement.setAttribute("type", getWrapperBOName() + ":" + getWrapperBOName());
		}
			
		writeXml(targetFile, doc);
	}

}
