/* IBM Confidential
 * OCO Source Materials
 * 5724-N40, 5724-N41, 5724-N42, 5724-N43, 5724-L77, 5724-L78, 5724-L79, 5724-L80, 5724-L81, 5724-L82
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.migration.sap.wbi_to_v610;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ibm.j2ca.migration.ArtifactChangeArguments;
import com.ibm.j2ca.migration.Change;
import com.ibm.j2ca.migration.IChangeArguments;
import com.ibm.j2ca.migration.MigrationException;
import com.ibm.j2ca.migration.changedata.wbi.ConvertWBIASItoJCAASI;
import com.ibm.j2ca.migration.internal.changes.wbi.ConvertWBIASItoJCAChange;
import com.ibm.j2ca.migration.sap.v602_to_v610.MigrationMessages;
import com.ibm.j2ca.migration.util.AttributeMap;
import com.ibm.j2ca.migration.util.SearchHelper;

public class DeleteRenameBOForBAPIRSChange extends Change {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2008."; //$NON-NLS-1$
	
	private IFile file;
	protected ArrayList<IFile> topLevelBOs = null;
	private String BAPI_RS_FK_Value = null;
	
	public DeleteRenameBOForBAPIRSChange(IFile file, DeleteRenameBOForBAPIRS changeData) {
		super(changeData);
		this.file = file;
	}
	
	public DeleteRenameBOForBAPIRS getChangeData() { return (DeleteRenameBOForBAPIRS) super.getChangeData(); }
	
	public String getChangeDetails() {
		return MigrationMessages.DeleteRenameBOForBAPIRSChange_Description;
	}
	
	public IChangeArguments getChangeArguments() {
		ArtifactChangeArguments argument = new ArtifactChangeArguments(file);
		return argument;
	}
	
	public void perform(IProgressMonitor pm) throws Exception {
		if(file.exists()) {
			DOMParser parser = new DOMParser();
			parser.parse(file.getLocation().toString());			
			Document doc = parser.getDocument();
			
			if (getChangeData().sapAPI.equalsIgnoreCase("BAPIRS") && isBAPI_RS_Wrap(file)) {
				//Find out Query BAPI name
				String queryBAPIName = null, queryBAPIType = null;
				NodeList elements = doc.getElementsByTagNameNS("*", "element");
				if(elements!=null) 
				  for(int i=0; i<elements.getLength(); i++) {
					Element element = (Element) elements.item(i);
					if(element.hasAttribute("name") && element.getAttribute("name").equalsIgnoreCase("BAPI_Query")) {
						queryBAPIName = getBAPIName(element); 
						queryBAPIType = getBAPIType(element);
					}
				}
				
				//Find Query BAPI file
				IFile queryBAPIFile = null;
				if(queryBAPIName!=null && queryBAPIType!=null) {
					queryBAPIFile = getFileByImportName(file, doc, queryBAPIName, queryBAPIType);
				}
				//Rename the Query BAPI File to "QueryBO.xsd"
				if(queryBAPIFile!=null && queryBAPIFile.exists()) {
					File queryFile = new File(queryBAPIFile.getLocation().toString());
					String path = queryBAPIFile.getLocation().toString();
					if(path.indexOf(queryBAPIFile.getName())>=0)
					   path = path.substring(0, path.indexOf(queryBAPIFile.getName()));
					if(queryFile.exists())
						queryFile.renameTo(new File(path+"QueryBO.xsd"));
				}
				//Delete the BAPI RS WBI Old Wrap BO
				File topLevelBO = new File(file.getLocation().toString());
				topLevelBO.delete();
				file.getProject().refreshLocal(IResource.DEPTH_INFINITE, pm);
			}
		}
	}
	
	IFile getFileByImportName(IFile file, Document doc, String name, String type) throws MigrationException {

		NodeList schemas = doc.getElementsByTagNameNS("*", "schema");
		String namespace = null;
		if(schemas!=null && schemas.getLength()>0) {
			Element schema = (Element) schemas.item(0);
			String attr = null;
			if(type.indexOf(":")>=0)
				attr = type.substring(0, type.indexOf(":"));
			if(attr!=null && schema.hasAttribute("xmlns:"+attr))
				namespace = schema.getAttribute("xmlns:"+attr);
		}
		NodeList imports = doc.getElementsByTagNameNS("*", "import");
		if(imports!=null && namespace!=null)
			for(int i=0; i<imports.getLength(); i++) {
				Element oneImport = (Element) imports.item(i);
				if(oneImport.hasAttribute("namespace") && oneImport.getAttribute("namespace").equals(namespace)) {
					if(oneImport.hasAttribute("schemaLocation")) {
						String filename = oneImport.getAttribute("schemaLocation");
						ArrayList<IFile> files = SearchHelper.getMigratableBOFiles(file.getProject());
						for(IFile oneFile : files) {
							if(oneFile.getName().equalsIgnoreCase(filename))
									return oneFile;
						}
					}
				}
			}
		return null;
	}
	
	
	String getBAPIType(Element element) {
		if(element.hasAttribute("type"))
			return element.getAttribute("type");
		return null;
	}
	
	String getBAPIName(Element element) {
		NodeList appSpecificInfos = element.getElementsByTagNameNS("*", "appSpecificInfo");
		if(appSpecificInfos!=null && appSpecificInfos.getLength()>0) {
			Element appSpecificInfo = (Element) appSpecificInfos.item(0);
			String value = appSpecificInfo.getTextContent();
			if(value!=null && value.length()>0) {
				String[] values = value.split(getChangeData().keyPairsSplitRegEx);
				for(String oneValue: values) {
					if(oneValue.toLowerCase().startsWith("bapi=")) {
						return oneValue.substring(5);
					}
				}
			}
		}
		return null;
	}
	
	boolean isBAPI_RS_Wrap(IFile file) throws SAXException, IOException {
		boolean ret = false;
		int count = 0;
		
		DOMParser parser = new DOMParser();
		parser.parse(file.getLocation().toString());
		Document doc = parser.getDocument();
		
		NodeList elements = doc.getElementsByTagNameNS("*", "element");
		if(elements!=null)
		   for(int i=0; i<elements.getLength(); i++) {
			  Element element = (Element) elements.item(i);
			  if(element.hasAttribute("name"))
				  if(element.getAttribute("name").equalsIgnoreCase("BAPI_Query"))
					  count++;
				  else if(element.getAttribute("name").equalsIgnoreCase("BAPI_Result"))
					  count++;
		   }
		if(count>=2) ret = true;
		return ret;
	}
	

}
