/* IBM Confidential
 * OCO Source Materials
 * 5724-L79
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.migration.sap.wbi_to_v610;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;

import org.eclipse.core.resources.IProject;

import com.ibm.j2ca.migration.MigrationException;
import com.ibm.j2ca.migration.ModuleMigrationTask;
import com.ibm.j2ca.migration.changedata.AddBOImport;
import com.ibm.j2ca.migration.changedata.ChangeConnectionAttributeValue;
import com.ibm.j2ca.migration.changedata.ChangeESBBindingAttribute;
import com.ibm.j2ca.migration.changedata.ModuleChangeData;
import com.ibm.j2ca.migration.changedata.RenameAdapter;
import com.ibm.j2ca.migration.changedata.ServiceChangeData.ServiceType;
import com.ibm.j2ca.migration.changedata.wbi.ConvertWBIASItoJCAASI;
import com.ibm.j2ca.migration.changedata.wbi.ConvertWBIElements;
import com.ibm.j2ca.migration.changedata.wbi.ConvertWBINamespaces;
import com.ibm.j2ca.migration.changedata.wbi.ConvertWBIWSDLExport;
import com.ibm.j2ca.migration.changedata.wbi.ConvertWBIWSDLImport;
import com.ibm.j2ca.migration.changedata.wbi.CreateComponent;
import com.ibm.j2ca.migration.changedata.wbi.CreateJavaInputClass;
import com.ibm.j2ca.migration.changedata.wbi.CreateJavaOutputClass;
import com.ibm.j2ca.migration.changedata.wbi.CreateMethodBindingsExport;
import com.ibm.j2ca.migration.changedata.wbi.CreateMethodBindingsImport;
import com.ibm.j2ca.migration.changedata.wbi.CreateWSDLExport;
import com.ibm.j2ca.migration.changedata.wbi.ImportWBIPropertiesWithDecrypt;
import com.ibm.j2ca.migration.changedata.wbi.RemoveBOAttributes;
import com.ibm.j2ca.migration.changedata.wbi.RemoveWBIASIData;
import com.ibm.j2ca.migration.changedata.wbi.RemoveWBIMethodBindings;
import com.ibm.j2ca.migration.changedata.wbi.RenameRAType;
import com.ibm.j2ca.migration.changedata.wbi.SetServiceTarget;
import com.ibm.j2ca.migration.changedata.wbi.UpdateJavaClass;
import com.ibm.j2ca.migration.util.AttributeMap;
import com.ibm.j2ca.migration.util.StringList;

public class SAPModuleMigrationTask extends ModuleMigrationTask {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2008."; //$NON-NLS-1$
	
	public static final String SAP_BUNDLE_NAME = "com.ibm.j2ca.migration.sap"; //$NON-NLS-1$
	public static final String BASE_BUNDLE_NAME = "com.ibm.j2ca.migration";	//$NON-NLS-1$
	protected static final String ADAPTER_NAME = "IBM WebSphere Adapter for SAP Software"; 
	protected static final String NAMESPACE_URI = "http://www.ibm.com/xmlns/prod/websphere/j2ca/sap";
	protected String BO_TYPE_METADATA_NODENAME = "sapBAPIBusinessObjectTypeMetadata";
	protected String ATTRIBUTE_TYPE_METADATA_NODENAME = "sapBAPIPropertyTypeMetadata";
	protected static final String NAMESPACE_PREFIX = "sapasi";
	protected static final String KEY_PAIRS_SPLIT_REGEX = ";";
	protected static final String VALUE_SPLIT_REGEX = ":";	
	protected String sapAPI = "";
	protected LinkedHashMap sapProjectAPIMap = null;
	protected String interactionSpec = "com.ibm.j2ca.sap.SAPBapiInteractionSpec";
	protected String activationSpec = "com.ibm.j2ca.sap.SAPActivationSpecWithXid";
	
	protected static AttributeMap<String> ASI_MAP = new AttributeMap<String>(AttributeMap.Type.IGNORE_CASE);
	static {
		ASI_MAP.put("isForeignKey", "ForeignKey");
		ASI_MAP.put("isKey", "PrimaryKey");
		ASI_MAP.put("maxLength", "MaxLength");
	}
	
	protected static AttributeMap<String> SUPPORTED_IMPORT_VERB_MAP = new AttributeMap<String>(AttributeMap.Type.NORMAL);
	static {
		SUPPORTED_IMPORT_VERB_MAP.put("Create");
		SUPPORTED_IMPORT_VERB_MAP.put("Update");
		SUPPORTED_IMPORT_VERB_MAP.put("Delete");
		SUPPORTED_IMPORT_VERB_MAP.put("Retrieve");
		//SUPPORTED_IMPORT_VERB_MAP.put("Retrieve", "RetrieveAll");
		//SUPPORTED_IMPORT_VERB_MAP.put("Exists");
	}
	
	protected static AttributeMap<String> SUPPORTED_EXPORT_VERB_MAP = new AttributeMap<String>(AttributeMap.Type.NORMAL);
	static {
		SUPPORTED_EXPORT_VERB_MAP.put("Create");
		SUPPORTED_EXPORT_VERB_MAP.put("Update");
		SUPPORTED_EXPORT_VERB_MAP.put("Delete");
	}
	
	protected static StringList OBSOLETE_ATTRIBUTES = new StringList();
	static {
		OBSOLETE_ATTRIBUTES.add("version");
		OBSOLETE_ATTRIBUTES.add("delta");
		OBSOLETE_ATTRIBUTES.add("locale");
	}
	
	protected static AttributeMap<String> IMPORT_PROPERTY_MAP = new AttributeMap<String>(AttributeMap.Type.IGNORE_CASE);
	static {
		IMPORT_PROPERTY_MAP.put("CharacterEncoding", "Codepage");
		IMPORT_PROPERTY_MAP.put("ApplicationPassword", "password");
		IMPORT_PROPERTY_MAP.put("ApplicationUserName", "userName");
		IMPORT_PROPERTY_MAP.put("LogFileName", "LogFileName");
		IMPORT_PROPERTY_MAP.put("Client", "Client");		
		IMPORT_PROPERTY_MAP.put("Hostname", "ApplicationServerHost");
		IMPORT_PROPERTY_MAP.put("Language", "Language");
		IMPORT_PROPERTY_MAP.put("Sysnr", "SystemNumber");
		IMPORT_PROPERTY_MAP.put("Group", "LogonGroup");
		IMPORT_PROPERTY_MAP.put("SAPSystemID", "SAPSystemID");
		IMPORT_PROPERTY_MAP.put("gwService", "GatewayService");
		IMPORT_PROPERTY_MAP.put("RfcTraceOn", "RFCTraceOn");
		IMPORT_PROPERTY_MAP.put("ABAPDebug", "ABAPDebug");
	}
	
	protected static AttributeMap<String> EXPORT_PROPERTY_MAP = new AttributeMap<String>(AttributeMap.Type.IGNORE_CASE);
	static {
		EXPORT_PROPERTY_MAP.put("CharacterEncoding", "Codepage");
		EXPORT_PROPERTY_MAP.put("ApplicationPassword", "password");
		EXPORT_PROPERTY_MAP.put("ApplicationUserName", "userName");
		EXPORT_PROPERTY_MAP.put("Client", "Client");		
		EXPORT_PROPERTY_MAP.put("Hostname", "ApplicationServerHost");
		EXPORT_PROPERTY_MAP.put("Language", "Language");
		EXPORT_PROPERTY_MAP.put("Sysnr", "SystemNumber");
		EXPORT_PROPERTY_MAP.put("Group", "LogonGroup");
		EXPORT_PROPERTY_MAP.put("SAPSystemID", "SAPSystemID");
		EXPORT_PROPERTY_MAP.put("gwService", "GatewayService");
		EXPORT_PROPERTY_MAP.put("RfcProgramId", "RfcProgramID");
		EXPORT_PROPERTY_MAP.put("RfcTraceOn", "RFCTraceOn");
		EXPORT_PROPERTY_MAP.put("NumberOfListeners", "NumberOfListeners");
		EXPORT_PROPERTY_MAP.put("ALEUpdateStatus", "AleUpdateStatus");
		EXPORT_PROPERTY_MAP.put("ALESelectiveUpdate", "AleSelectiveUpdate");
		EXPORT_PROPERTY_MAP.put("ALEStatusMsgCode", "AleStatusMsgCode");
		EXPORT_PROPERTY_MAP.put("ALESuccessCode", "AleSuccessCode");
		EXPORT_PROPERTY_MAP.put("ALEFailureCode", "AleFailureCode");
		EXPORT_PROPERTY_MAP.put("ALESuccessText", "AleSuccessText");
		EXPORT_PROPERTY_MAP.put("ALEFailureText", "FailureText");
	}
	
	protected static AttributeMap<String> RESPONSE_MAP = new AttributeMap<String>(AttributeMap.Type.IGNORE_CASE);
	static {
		//RESPONSE_MAP.put("Exists", "ExistsResult:ExistsResult");
	}
	
	protected static AttributeMap<String> NEW_IMPORT_PROPERTIES = new AttributeMap<String>(AttributeMap.Type.IGNORE_CASE);
	static {
		NEW_IMPORT_PROPERTIES.put("maxRecords", "1");
	}

	public ArrayList<ModuleChangeData> getChangeData() throws MigrationException {
		
		try {
			sapProjectAPIMap = SAPMigrationUtil.getSAPProjectAPIMap(getMigrationContext());
		} catch (Exception e) {
			throw new MigrationException(e);
		}
		
		Collection<IProject> projects = getMigrationContext().getDependentModules();
		for(IProject project : projects) {
			sapAPI = (String) sapProjectAPIMap.get(project);
			if(sapAPI == null || 
			    !(sapAPI.equalsIgnoreCase("BAPI") ||
			      sapAPI.equalsIgnoreCase("BAPITXN") || 
			      sapAPI.equalsIgnoreCase("BAPIRS") ||
			      sapAPI.equalsIgnoreCase("ALE")))
		        //Only support BAPI and ALE interfaces
				throw new MigrationException(new Exception("Only BAPI and ALE interfaces are supported for migrating from WBI SAP Adapter to JCA SAP Adapter."));
		}
		
		if(sapAPI.startsWith("BAPI")) {
			BO_TYPE_METADATA_NODENAME = "sapBAPIBusinessObjectTypeMetadata";
			ATTRIBUTE_TYPE_METADATA_NODENAME = "sapBAPIPropertyTypeMetadata";
			interactionSpec = "com.ibm.j2ca.sap.SAPBapiInteractionSpec";
			
		} else {
			//...
		}
		
		ArrayList<ModuleChangeData> changeDataList = new ArrayList<ModuleChangeData>();
		
		changeDataList.addAll(getBOChangeData());
		
		changeDataList.add(new RenameAdapter(ADAPTER_NAME, "SIB JMS Resource Adapter", new ServiceType [] {ServiceType.IMPORT, ServiceType.EXPORT}));
		changeDataList.add(new RenameRAType("com.ibm.j2ca.sap.SAPResourceAdapter", "com\\.ibm\\.ws\\.sib\\.api\\.jmsra\\.impl\\.JmsJcaResourceAdapterImpl", new ServiceType [] {ServiceType.IMPORT, ServiceType.EXPORT}));
		//dataBindingType is changed in v602_t0_v610 steps
		//changeDataList.add(new ChangeESBBindingAttribute("dataBindingType", "com.ibm.j2ca.jdbc.emd.runtime.SAPDataBindingGenerator", new ServiceType [] {ServiceType.IMPORT, ServiceType.EXPORT}));
		changeDataList.add(new RemoveWBIMethodBindings(new ServiceType [] {ServiceType.IMPORT, ServiceType.EXPORT}));
		
		//Export Change needs to be called first, before creating the wrapper bo and bg
		changeDataList.addAll(getExportServiceChangeData());
		changeDataList.addAll(getImportServiceChangeData());
		
		com.ibm.j2ca.migration.sap.v602_to_v610.SAPModuleMigrationTask task_v602_to_v610 = new com.ibm.j2ca.migration.sap.v602_to_v610.SAPModuleMigrationTask(true);
		task_v602_to_v610.sapAPI = sapAPI;
		changeDataList.addAll(task_v602_to_v610.getChangeData());
		
		return changeDataList;
	}
	
	protected ArrayList<ModuleChangeData> getBOChangeData() {
		ArrayList<ModuleChangeData> changeDataList = new ArrayList<ModuleChangeData>();
		
		//changeDataList.add(new AddBOVerb("RetrieveByContent"));
		
		//Add the Wrapper for the top-level BO
		//changeDataList.add(new CreateBOWrapper("/resources/wbi_to_v610/bo/WrapperTemplate.xsd", BUNDLE_NAME, NAMESPACE_URI, true));

		ConvertWBIASItoJCAASI convertWBIASItoJCAASI = new ConvertWBIASItoJCAASI();
		convertWBIASItoJCAASI.namespacePrefix = NAMESPACE_PREFIX;
		convertWBIASItoJCAASI.namespaceURI = NAMESPACE_URI;
		convertWBIASItoJCAASI.boMetadataNodeName = BO_TYPE_METADATA_NODENAME;
		convertWBIASItoJCAASI.attributeMetadataNodeName = ATTRIBUTE_TYPE_METADATA_NODENAME;
		convertWBIASItoJCAASI.asiMap = ASI_MAP;
		convertWBIASItoJCAASI.keyPairsSplitRegEx = KEY_PAIRS_SPLIT_REGEX;
		convertWBIASItoJCAASI.valueSplitRegEx = VALUE_SPLIT_REGEX;
		changeDataList.add(convertWBIASItoJCAASI);
		
		ConvertWBIASItoJCAASIForSAP convertWBIASItoJCAASIForSAP = new ConvertWBIASItoJCAASIForSAP();
		convertWBIASItoJCAASIForSAP.namespacePrefix = NAMESPACE_PREFIX;
		convertWBIASItoJCAASIForSAP.namespaceURI = NAMESPACE_URI;
		convertWBIASItoJCAASIForSAP.boMetadataNodeName = BO_TYPE_METADATA_NODENAME;
		convertWBIASItoJCAASIForSAP.attributeMetadataNodeName = ATTRIBUTE_TYPE_METADATA_NODENAME;
		convertWBIASItoJCAASIForSAP.asiMap = ASI_MAP;
		convertWBIASItoJCAASIForSAP.keyPairsSplitRegEx = KEY_PAIRS_SPLIT_REGEX;
		convertWBIASItoJCAASIForSAP.valueSplitRegEx = VALUE_SPLIT_REGEX;
		convertWBIASItoJCAASIForSAP.sapAPI = sapAPI;
		changeDataList.add(convertWBIASItoJCAASIForSAP);
		
		/*
		ConvertWBIASItoJCAProperties convertWBIASItoJCAProperties = new ConvertWBIASItoJCAProperties();
		convertWBIASItoJCAProperties.namespacePrefix = NAMESPACE_PREFIX;
		convertWBIASItoJCAProperties.namespaceURI = NAMESPACE_URI;
		convertWBIASItoJCAProperties.boMetadataNodeName = BO_TYPE_METADATA_NODENAME;
		convertWBIASItoJCAProperties.attributeMetadataNodeName = ATTRIBUTE_TYPE_METADATA_NODENAME;
		convertWBIASItoJCAProperties.asiPropertyMap = ASI_PROPERTY_MAP;
		convertWBIASItoJCAProperties.keyPairsSplitRegEx = KEY_PAIRS_SPLIT_REGEX;		
		changeDataList.add(convertWBIASItoJCAProperties);		
		*/
		
		ConvertWBIElements convertWBIElement = new ConvertWBIElements();
		convertWBIElement.isAddingType = false;
		changeDataList.add(convertWBIElement);
		
		changeDataList.add(new ConvertWBINamespaces(NAMESPACE_PREFIX, NAMESPACE_URI));
		changeDataList.add(new RemoveWBIASIData());
		
		//Only QISS API has BO Container
		//changeDataList.add(new CreateBOContainer("/resources/wbi_to_v610/bo/ContainerTemplate.xsd", MIGRATION_BUNDLE_NAME, false));
		changeDataList.add(new AddBOImport("SapBusinessObjectMetadata\\.xsd", ".*\\.xsd", ".*((BG)|(Container))\\.xsd"));
		changeDataList.add(new RemoveBOAttributes(OBSOLETE_ATTRIBUTES));
		
		if(sapAPI.equalsIgnoreCase("BAPIRS")) {
			changeDataList.add(new DeleteRenameBOForBAPIRS(sapAPI));
		}
		return changeDataList;
	}
	
	protected ArrayList<ModuleChangeData> getImportServiceChangeData() {
		ArrayList<ModuleChangeData> changeDataList = new ArrayList<ModuleChangeData>();
		
		changeDataList.add(new ChangeESBBindingAttribute("xsi:type", "EISImportBinding", new ServiceType [] {ServiceType.IMPORT}));
		changeDataList.add(new ChangeConnectionAttributeValue("type", "com.ibm.j2ca.sap.SAPManagedConnectionFactory", new ServiceType [] {ServiceType.IMPORT}));
		changeDataList.add(new ChangeConnectionAttributeValue("interactionType", interactionSpec, new ServiceType [] {ServiceType.IMPORT}));		
		changeDataList.add(new ImportWBIPropertiesWithDecrypt(IMPORT_PROPERTY_MAP, new ServiceType [] {ServiceType.IMPORT}));
		
		if(sapAPI.equalsIgnoreCase("BAPI") || sapAPI.equalsIgnoreCase("BAPIRS")) {
			//Add the Wrapper for the top-level BOs
			changeDataList.add(new CreateBOWrapperForSAP("/resources/wbi_to_v610/bo/BAPIWrapper.xsd", SAP_BUNDLE_NAME, NAMESPACE_URI, true, NAMESPACE_URI, NAMESPACE_PREFIX));
			//Add the WrapperBG for the top-level BOs
			changeDataList.add(new CreateBOWrapperBGForSAP("/resources/wbi_to_v610/bo/BAPIWrapperBG.xsd", SAP_BUNDLE_NAME, NAMESPACE_URI, true));
		}
		
		CreateMethodBindingsImport createMethodBindingsImport = new CreateMethodBindingsImport(SUPPORTED_IMPORT_VERB_MAP);
		if(sapAPI.equalsIgnoreCase("BAPI") || sapAPI.equalsIgnoreCase("BAPIRS"))
			createMethodBindingsImport.isTopLevelBOChanged = true;
		changeDataList.add(createMethodBindingsImport);
		
		ConvertWBIWSDLImport convertWBIWSDLImport = new ConvertWBIWSDLImport(SUPPORTED_IMPORT_VERB_MAP, RESPONSE_MAP);
		if(sapAPI.equalsIgnoreCase("BAPI") || sapAPI.equalsIgnoreCase("BAPIRS"))
			convertWBIWSDLImport.isDeleteOldImports = false;
		//    convertWBIWSDLImport.isRemoveOldElementMessageAndOperationInWSDL = true;
		changeDataList.add(convertWBIWSDLImport);
		//changeDataList.add(new CreateMethodBindingProperties("RetrieveAll", NEW_IMPORT_PROPERTIES, new ServiceType [] {ServiceType.IMPORT}));
		
		String Output_Processing_JavaFile = "/resources/wbi_to_v610/Output_Processing.java";
		if(sapAPI.equalsIgnoreCase("BAPI") || sapAPI.equalsIgnoreCase("BAPIRS")) {
			Output_Processing_JavaFile = "/resources/wbi_to_v610/BAPI/Output_Processing.java";
		}
		CreateJavaOutputClass createJavaOutputClass = new CreateJavaOutputClass(Output_Processing_JavaFile, SAP_BUNDLE_NAME, SUPPORTED_IMPORT_VERB_MAP);
		if(sapAPI.equalsIgnoreCase("BAPI") || sapAPI.equalsIgnoreCase("BAPIRS")) {
		   createJavaOutputClass.isTopLevelBOChanged = true;
		   //createJavaOutputClass.isDynamicVerbMap = true;
		   createJavaOutputClass.inputTypeChanged = true;
		   createJavaOutputClass.originalTopLevelBOs = SAPMigrationUtil.projectTopLevelBOMap;
		}
		createJavaOutputClass.namespaceURI = NAMESPACE_URI;
		changeDataList.add(createJavaOutputClass);
		
		return changeDataList;
	}
	
	protected ArrayList<ModuleChangeData> getExportServiceChangeData() {
		ArrayList<ModuleChangeData> changeDataList = new ArrayList<ModuleChangeData>();
		
		changeDataList.add(new ChangeESBBindingAttribute("xsi:type", "EISExportBinding", new ServiceType [] {ServiceType.EXPORT}));
		changeDataList.add(new ChangeConnectionAttributeValue("type", activationSpec, new ServiceType [] {ServiceType.EXPORT}));
		changeDataList.add(new ChangeConnectionAttributeValue("listenerType", "com.ibm.j2ca.base.ExtendedInboundListener", new ServiceType [] {ServiceType.EXPORT}));
		changeDataList.add(new ChangeConnectionAttributeValue("selectorType", "com.ibm.j2ca.extension.emd.runtime.StructuredDataFunctionSelector", new ServiceType [] {ServiceType.EXPORT}));		
		changeDataList.add(new ImportWBIPropertiesWithDecrypt(EXPORT_PROPERTY_MAP, new ServiceType [] {ServiceType.EXPORT}));
		
		CreateMethodBindingsExport createMethodBindingsExport = new CreateMethodBindingsExport(SUPPORTED_EXPORT_VERB_MAP);
		//if(sapAPI.startsWith("BAPI"))
		//	createMethodBindingsExport.isTopLevelBOChanged = true;
		changeDataList.add(createMethodBindingsExport);
		
		//Create the interface WSDL file and update the references. 
		changeDataList.add(new CreateWSDLExport("/resources/wbi_to_v620/wsdl/Inbound_PortType.wsdl",BASE_BUNDLE_NAME));

		ConvertWBIWSDLExport convertWBIWSDLExport = new ConvertWBIWSDLExport(SUPPORTED_EXPORT_VERB_MAP);
		//if(sapAPI.startsWith("BAPI"))
		//	convertWBIWSDLExport.isRemoveOldMessageAndOperationInWSDL = true;
		changeDataList.add(convertWBIWSDLExport);
		
		//changeDataList.add(new SetConnectionPropertyValue("FilterFutureEvents", "_true", "FilterFutureEvents", "false", new ServiceType [] {ServiceType.EXPORT}));
		//changeDataList.add(new SetConnectionPropertyValue("FilterFutureEvents", "false", "FilterFutureEvents", "true", new ServiceType [] {ServiceType.EXPORT}));
		//changeDataList.add(new SetConnectionPropertyValue("FilterFutureEvents", "true", "FilterFutureEvents", "_true", new ServiceType [] {ServiceType.EXPORT}));
		
		changeDataList.add(new CreateJavaInputClass("/resources/wbi_to_v610/Input_Async_Processing.java", SAP_BUNDLE_NAME, SUPPORTED_EXPORT_VERB_MAP));
		changeDataList.add(new CreateComponent("/resources/wbi_to_v610/Input_Async_Processing.component", SAP_BUNDLE_NAME, "Input_Async\\.export"));
		changeDataList.add(new SetServiceTarget("Input_Async_Processing", "Input_Async\\.export", new ServiceType [] {ServiceType.EXPORT}));
		
		//Inbound_Sync is not supported for JCA PeopleSoft adapter
		//But still leave the interface in the module
		//Add some codes into Input_Processing.java for removing the compile error on it
		changeDataList.add(new UpdateJavaClass("Input_Processing.java", SUPPORTED_EXPORT_VERB_MAP));

		return changeDataList;
	}
}
